#!/usr/bin/perl -w
# minordomo.pl - This is minordomo, a minimalistic mailing list manager
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version. And also
# please DO NOT REMOVE my name, and give me a CREDIT when you use
# whole or a part of this program in an other program.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
use Socket;
use POSIX qw(strftime);
use strict;

# You may wish to edit this if you are using a different config file
my $config = "/etc/minordomo.conf";

# The version of minordomo
my $version = "minordomo 0.7";

# Set up default values
my $list = '';
my $smtp_server = 'localhost';
my $directory = '/var/lib/minordomo';
my $admin = 'root@localhost';
my $domain = 'localdomain.com';
my $archive = '';
my $list_cmd = '';
my $open_lists = '';
my $closed_lists = '';
my $subject_pre = '';
my $url = '';
my $message = '';
my $set_re_to = 'no';
my $max_size = 0;

# Read the config file
do_conf($config);

# Get the list name
$list = $ARGV[0];

# Read in the message and find out who it's from
while (<STDIN>){
 $message .= $_;
}
my ($header, $mess) = split(/\n\n/, $message, 2);
my @header = split (/\n/, $header);
my $from = '';
foreach (@header){ if (($_ =~ /^from:/i) && ($from eq '')){ $from= $_; } }
$from =~ s/^from: //ig;
if ($from =~ /<.*>/) { $from =~ s/.*<(.*\@.*\..*)>/$1/; }
elsif ($from =~ /\(.*\)/) { $from =~ s/(.*\@.*\..*).*/$1/g; }

# For a message to a list
unless ($list eq ''){
 mkdir "$directory/$list", 0755;
 do_conf("$directory/$list/config");

 # Get the subscriber list
 my $listmembers = '';
 open LIST, "$directory/$list/list";
 while (<LIST>){ $listmembers .= $_; }
 close LIST;

 # Find out if they are a list member
 $listmembers =~ s/\r//g;
 my @listmembers = split (/\n/, $listmembers);
 my $send_ok = 'no';
 foreach (@listmembers){ if ($_ eq $from) { $send_ok = 'yes'; } }

 # Or if this is an open list
 foreach (split(/ /, $open_lists)){ if (($_ eq $list)||($_ eq "*")) { $send_ok = 'yes'; } }

 # But not if it's a closed list!
 foreach (split(/ /, $closed_lists)){ if (($_ eq $list)||($_ eq "*")) { $send_ok = 'no'; } }

 # Unless they are the admin, then it's OK
 if ($from eq $admin) { $send_ok = 'yes'; }

 # Read in the footer
 my $footer = '';
 my $footfile = $directory . "/" . $list . "/footer";
 if (-s $footfile){
  open FOOT, "$footfile";
  while (<FOOT>){ $footer .= $_; }
  close FOOT;
 }

 # Do footer expansion
 $footer =~ s/\\a/$admin/ig;
 $footer =~ s/\\d/$domain/ig;
 $footer =~ s/\\l/$list/ig;
 $footer =~ s/\\u/$url/ig;

 # Send the message
 my $to = '';
 my $noarchive = 0;
 if ($send_ok eq 'yes' && (length($message) < $max_size || $max_size == 0)){
  foreach (@listmembers){
   if ($to eq ''){ $to = $_; }
   else { $to .= " $_"; }
  }

  # Add the subject prefix
  if ($subject_pre ne '') {
#DEBUG
$footer .= "length: " . length($message) . "\nmax: $max_size";
   foreach(@header) {
    if ($_ =~ /^reply-to:/i && $set_re_to =~ /^yes$/ig) {
     $_ =~ s/.*/Reply-To: $list\@$domain/;
     $set_re_to = 'no';
    } elsif (($_ =~ /^subject:/i) && !($_ =~ /$subject_pre/)) {
     $subject_pre =~ s/\\([\[|\]])/$1/g;
     $_ =~ s/(: )/$1$subject_pre /;
    }
   }
   $header = join("\r\n", @header);
  }

  # Add various things to the header
  $header .= "\r\nX-Mailing-List: $list\@$domain\r\n";
  $header .= "X-Mailing-List-Server: $version\r\n";
  $header .= "X-Loop: $list\@$domain\r\n";
  $header .= "Precedence: bulk\r\n";
  $header .= "Reply-To: $list\@$domain\r\n" if $set_re_to =~ /^yes$/ig;
  
  # Get archive information
  foreach (split (/ /, $archive)) {
   if (($_ eq $list) || ($_ eq "*")){
    my ($messno, $archfile) = get_archive($list);
    if ($url ne ''){
     $url =~ s/[\r|\n]//g; $header .= "X-Archive-URL: $url\r\n";
    }
    $header .= "X-Archive-ID: $messno\r\n";
   }
  }

  # Finish preparation
  $header .= "\r\n";
  $message = "$header$mess\r\n$footer";

  send_mesg($to, $message);
 } elsif ($send_ok eq 'yes') {
  $noarchive = 1;
  sys_mess($from, "Too big", "Sorry, your message was larger than the maximum size for this list ($list\@$domain).");
 } else {
  # This person can't send to the list
  $noarchive = 1;
  sys_mess($from, "Not allowed", "This e-mail address ($from) is not allowed to post to this list ($list\@$domain).");
 }

 # Archive the message
 foreach (split (/ /, $archive)) {
  if (($_ eq $list) || ($_ eq "*") && !($noarchive)){
   my ($messno, $archfile) = get_archive($list);
   open ARCHIVE, ">$archfile";
   print ARCHIVE $message;
   close ARCHIVE;
  }
 }

# For a message to minordomo
} else {

 # Get the command from the subject
 @header = split(/\n/, $header);
 my $subject = '';
 foreach (@header){
  if (($_ =~ /^subject/i) && ($subject eq '')){ $subject = $_; }
 }
 my ($junk, $command) = split (/: /, $subject, 2);
 undef $junk;
 $list = '';
 my $cmd = '';
 ($cmd, $list) = split (/ /, $command, 2);

 # Read the list specific config file
 do_conf("$directory/$list/config");

 # Execute the command
 if ($cmd eq 'subscribe') {
  subscribe($from, $list);
 } elsif ($cmd eq 'unsubscribe') {
  unsubscribe($from, $list);
 } elsif ($cmd eq 'info') {
  info($from, $list);
 } elsif ($cmd eq 'list') {
  list($from, $list);
 } else {
  $mess .= "Unrecongnized command $cmd.";
  send_mesg($from, $mess);
 }
} 

###############################################################################
# Send a message
# syntax - send_mesg($to, $message);
###########################################################################
sub send_mesg{
 # Open the socket
 my $proto = getprotobyname('tcp');
 socket(SOCK, AF_INET, SOCK_STREAM, $proto);
 my $iaddr = gethostbyname($smtp_server);
 my $port = getservbyname('smtp', 'tcp');
 my $sin = sockaddr_in($port, $iaddr);
 connect(SOCK, $sin);

 # Get the variables
 my $to = shift;
 $to =~s/,//g;
 my @tos = split(/ /, $to);
 my $message = shift;
 my $err = '';

 # SMTP session
 send SOCK, "HELO $domain\r\n", 0;
 recv SOCK, $err, 512, 0;
 send SOCK, "MAIL FROM: $admin\r\n", 0;
 recv SOCK, $err, 512, 0;
 foreach (@tos){ send SOCK, "RCPT TO: $_\r\n", 0; recv SOCK, $err, 512, 0; }
 send SOCK, "DATA\r\n", 0;
 recv SOCK, $err, 512, 0;
 send SOCK, "$message\r\n.\r\n", 0;
 recv SOCK, $err, 512, 0;
 send SOCK, "QUIT\r\n", 0;
 recv SOCK, $err, 512, 0;
 close SOCK;
} 

###########################################################################
# Gets the archive filename and message number
# Syntax - get_archive($list);
###########################################################################
sub get_archive{
 # Get some variables
 my $list = shift;
 my @date = localtime;
 my $year = 1900 + $date[5];
 my $month = 1 + $date[4];
 my $day = $date[3];

 # Make the proper directories
 mkdir "$directory/$list/archive", 0755;
 mkdir "$directory/$list/archive/$year", 0755;
 mkdir "$directory/$list/archive/$year/$month", 0755;
 mkdir "$directory/$list/archive/$year/$month/$day", 0755;

 # Find out the message number
 my $messno = "0";
 my $break = 'no';
 while ($break ne 'yes'){
  if (-e "$directory/$list/archive/$year/$month/$day/$messno"){ $messno++;
  } else { $break = 'yes'; }
 }
 my $file = "$directory/$list/archive/$year/$month/$day/$messno";

 # Return the number and filename
 return ($messno, $file);
}

###########################################################################
# Reads a configuration file
# Syntax - do_conf($config_filename);
###########################################################################
sub do_conf{
 # Get the file to read and initialize variables
 my $file = shift;
 my $directive = '';

 # Parse the thing
 open CONF, $file;
 while (<CONF>){
  if($_ =~ /^#/){
   # Comments start with a #
  } elsif ($_ =~ /^smtp_server/i){
   ($directive, $smtp_server) = split(/=/, $_);
   chomp $smtp_server;
  } elsif ($_ =~ /^directory/i){
   ($directive, $directory) = split(/=/, $_);
   chomp $directory;
  } elsif ($_ =~ /^admin/i){
   ($directive, $admin) = split(/=/, $_);
   chomp $admin;
  } elsif ($_ =~ /^domain/i){
   ($directive, $domain) = split(/=/, $_);
   chomp $domain;
  } elsif ($_ =~ /^archive/i){
   ($directive, $archive) = split(/=/, $_);
   chomp $archive;
  } elsif ($_ =~ /^list_cmd/i){
   ($directive, $list_cmd) = split(/=/, $_);
   chomp $list_cmd;
  } elsif ($_ =~ /^open_lists/i){
   ($directive, $open_lists) = split(/=/, $_);
   chomp $open_lists;
  } elsif ($_ =~ /^closed_lists/i){
   ($directive, $closed_lists) = split(/=/, $_);
   chomp $closed_lists;
  } elsif ($_ =~ /^subject_prefix/i){
   ($directive, $subject_pre) = split(/=/, $_);
   chomp $subject_pre;
  } elsif ($_ =~ /^set_re_to/i){
   ($directive, $set_re_to) = split(/=/, $_);
   chomp $set_re_to;
  } elsif ($_ =~ /^max_size/i){
   ($directive, $max_size) = split(/=/, $_);
   chomp $max_size;
  } elsif ($_ =~ /^url/i){
   ($directive, $url) = split(/=/, $_);
   chomp $url;
  }
 }
 close CONF;

 # These should be recognized as regular expressions
 $subject_pre =~ s/[\r|\n]//g;
 $subject_pre =~ s/([\[|\]])/\\$1/g;
}

###########################################################################
# For system messages
# Syntax - sys_mess($to, $subject, $message);
###########################################################################
sub sys_mess{
 my $to = shift;
 my $sub = shift;
 my $message = shift;
 my $mess = '';
 my $now = strftime("%a, %d %b %Y %T %z (%Z)",localtime);
 $mess .= "Date: $now\r\n";
 $mess .= "From: minordomo\@$domain\r\n";
 $mess .= "To: $to\r\n";
 $mess .= "Subject: $sub\r\n\r\n";
 $mess .= $message;
 send_mesg("$to, $admin", $mess);
}

###########################################################################
# Subscribe to a list
# Syntax - subscribe($member, $list);
###########################################################################
sub subscribe{
 my $member = shift;
 my $list = shift;
 my $dont_add = 0;
 my $message = '';

 # Subscribe to the list
 if ((-d "$directory/$list") && ($list ne '')) {

  my $listfile = '';
  # Add them to the file
  if (-s "$directory/$list/list") {
   open LISTFILE, "$directory/$list/list";
   while (<LISTFILE>) { $listfile .= $_ }
   close LISTFILE;
   if ($listfile =~ $member) { $dont_add = 1; }
   open LISTFILE, ">>$directory/$list/list";
  } else { open LISTFILE, ">$directory/$list/list"; }
  unless ($dont_add == 1) { print LISTFILE "$member\n"; }
  close LISTFILE;

  # Let them know it worked
  my $message = '';
  if ($dont_add == 1){
   $message = "You are already subscribed";
  } else {
   $message = "You have been successfully subscribed";
  }
  sys_mess($member, "subscribe $list", "$message to $list\@$domain.");
  unless ($dont_add ==1) {info($member, $list); }

 } else {
  # No such list here!
  sys_mess($member, "subscribe $list", "There is no such list $list here.");
 }
}

###########################################################################
# Unubscribe from a list
# Syntax - unsubscribe($member, $list);
###########################################################################
sub unsubscribe{
 my $member = shift;
 my $list = shift;

 if ((-d "$directory/$list") && ($list ne '')) {
  # Remove from the list
  my $listfile = '';
  open LISTFILE, "$directory/$list/list";
  while (<LISTFILE>) { $listfile .= $_; }
  close LISTFILE;
  $listfile =~ s/$member\n//g;
  $listfile =~ s/$member//g;
  $listfile =~ s/\r//g;
  open LISTFILE, ">$directory/$list/list";
  print LISTFILE $listfile;
  close LISTFILE;

  # It worked
  $mess = "You have been unsubscribed from $list\@$domain.";
  sys_mess($member, "unsubscribe $list", $mess);
 } else {
  # No such list
  $mess = "You are not subscribed to $list\@$domain.";
  sys_mess($member, "unsubscribe $list", $mess);
 }
}

###########################################################################
# List subscribers
# Syntax - list($towho, $list);
###########################################################################
sub list{
 my $to = shift;
 my $list = shift;
 my $now = strftime("%a, %d %b %Y %T %z (%Z)",localtime);
 my $mess = "Date: $now\r\nFrom: minordomo\@$domain\r\nTo: $to\r\n";
 $mess .= "Subject: Re: list $list\r\n\r\n";
 $mess .= "List follows. If blank, the admin has turned this off, or the list ";
 $mess .= "\r\nhas no subscribers.\r\n\r\n";

 foreach (split (/ /, $list_cmd)) {
  if (($_ eq $list) || ($_ eq "*") && (-e "$directory/$list/list")){
   open LIST, "$directory/$list/list";
   while (<LIST>) { $mess .= $_ };
   close LIST;
  }
 }
 send_mesg($to, $mess);
}

###########################################################################
# Get info for a list, or list available lists
# Syntax - list($to, $list);
###########################################################################
sub info{
 my $to = shift;
 my $list = shift;
 my $now = strftime("%a, %d %b %Y %T %z (%Z)",localtime);
 my $mess = "Date: $now\r\nFrom: minordomo\@$domain\r\nTo: $to\r\n";
 $mess .= "Subject: Re: info $list\r\n\r\n";

 if ((-d "$directory/$list") && ($list ne '')) {
  if (-s "$directory/$list/info"){
   # Read in the info file
   open INFO, "$directory/$list/info";
   while (<INFO>) { $mess .= $_ };
   close INFO;
  } else {
   # No info file!
   $mess .= "There is no information available for this list";
  }

 } elsif ($list ne '') {
  # No such list!
  $mess .= "There is no such list $list here. Send a message to minordomo\@$domain with a subject of 'info' (no quotes) for a list of available lists.";

 } else {
  # Basic commands
  $mess .= "This is a minordomo run list server. Commands are:\r\n\r\n";
  $mess .= "subscribe <list>:       adds your e-mail address to a list\r\n";
  $mess .= "unsubscribe <list>:     removes your address from a list\r\n";
  $mess .= "info [<list>]:          gives you information on a list\r\n";
  $mess .= "                        if <list> is omitted, gives this message";
  $mess .= "list <list>:            returns a list of subscribers to a\r\n";
  $mess .= "                        list, if enabled by the admin\r\n";
  $mess .= "\r\n\r\nAvailable lists are:\r\n";

  # List of lists
  open INFO, "$directory/lists";
  while (<INFO>) { $mess .= $_; }
  close INFO;
 }

 send_mesg($to, $mess);
}
